=== modified file 'NEWS'
--- NEWS	2007-02-22 14:28:19 +0000
+++ NEWS	2007-02-26 10:02:58 +0000
@@ -124,6 +124,9 @@
 
     * Fix problem with UNC paths on Windows 98. (Alexander Belchenko, #84728)
 
+    * Searching location of CA bundle for PyCurl in env variable (CURL_CA_BUNDLE),
+      and on win32 along the PATH. (Alexander Belchenko, #82086)
+
   TESTING:
 
     * New ``--first`` option to ``bzr selftest`` to run specified tests

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2007-02-15 17:54:17 +0000
+++ bzrlib/tests/__init__.py	2007-02-26 10:02:58 +0000
@@ -1784,6 +1784,7 @@
                    'bzrlib.tests.test_hashcache',
                    'bzrlib.tests.test_http',
                    'bzrlib.tests.test_http_response',
+                   'bzrlib.tests.test_https_ca_bundle',
                    'bzrlib.tests.test_identitymap',
                    'bzrlib.tests.test_ignores',
                    'bzrlib.tests.test_inv',

=== added file 'bzrlib/tests/test_https_ca_bundle.py'
--- bzrlib/tests/test_https_ca_bundle.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_https_ca_bundle.py	2007-02-26 10:02:58 +0000
@@ -0,0 +1,77 @@
+# Copyright (C) 2007 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Testing of bzrlib.transport.http.ca_bundle module"""
+
+import os
+import sys
+
+import bzrlib
+from bzrlib import osutils
+from bzrlib.tests import (
+    TestCaseInTempDir,
+    TestSkipped,
+    )
+from bzrlib.transport.http import ca_bundle
+
+
+class TestGetCAPath(TestCaseInTempDir):
+
+    def setUp(self):
+        TestCaseInTempDir.setUp(self)
+        new_env = {
+            'CURL_CA_BUNDLE': None,
+            'PATH': None,
+            }
+        self._old_env = {}
+        self.addCleanup(self._restore)
+        for name, value in new_env.iteritems():
+            self._old_env[name] = osutils.set_or_unset_env(name, None)
+
+    def _restore(self):
+        for name, value in self._old_env.iteritems():
+            osutils.set_or_unset_env(name, value)
+
+    def _make_file(self, in_dir='.'):
+        fname = os.path.join(in_dir, 'curl-ca-bundle.crt')
+        f = file(fname,'w')
+        f.write('spam')
+        f.close()
+
+    def test_found_nothing(self):
+        self.assertEqual('', ca_bundle.get_ca_path(use_cache=False))
+
+    def test_env_var(self):
+        osutils.set_or_unset_env('CURL_CA_BUNDLE', 'foo.bar')
+        self._make_file()
+        self.assertEqual('foo.bar', ca_bundle.get_ca_path(use_cache=False))
+
+    def test_in_cwd(self):
+        if sys.platform != 'win32':
+            raise TestSkipped('Searching in cwd implemented only for win32')
+        self._make_file()
+        shouldbe = os.path.join(os.getcwd(), 'curl-ca-bundle.crt')
+        self.assertEqual(shouldbe, ca_bundle.get_ca_path(use_cache=False))
+        
+    def test_in_path(self):
+        if sys.platform != 'win32':
+            raise TestSkipped('Searching in PATH implemented only for win32')
+        os.mkdir('foo')
+        in_dir = os.path.join(os.getcwd(), 'foo')
+        self._make_file(in_dir=in_dir)
+        osutils.set_or_unset_env('PATH', in_dir)
+        shouldbe = os.path.join(in_dir, 'curl-ca-bundle.crt')
+        self.assertEqual(shouldbe, ca_bundle.get_ca_path(use_cache=False))

=== modified file 'bzrlib/transport/http/_pycurl.py'
--- bzrlib/transport/http/_pycurl.py	2006-12-13 05:07:19 +0000
+++ bzrlib/transport/http/_pycurl.py	2007-02-26 10:02:58 +0000
@@ -25,6 +25,7 @@
 
 import os
 from cStringIO import StringIO
+import sys
 
 from bzrlib import (
     errors,
@@ -37,6 +38,7 @@
 from bzrlib.trace import mutter
 from bzrlib.transport import register_urlparse_netloc_protocol
 from bzrlib.transport.http import (
+    ca_bundle,
     _extract_headers,
     HttpTransportBase,
     _pycurl_errors,
@@ -79,6 +81,7 @@
 
     def __init__(self, base, from_transport=None):
         super(PyCurlTransport, self).__init__(base)
+        self.cabundle = ca_bundle.get_ca_path()
         if from_transport is not None:
             self._curl = from_transport._curl
         else:
@@ -230,6 +233,8 @@
         ua_str = 'bzr/%s (pycurl)' % (bzrlib.__version__,)
         curl.setopt(pycurl.USERAGENT, ua_str)
         curl.setopt(pycurl.FOLLOWLOCATION, 1) # follow redirect responses
+        if self.cabundle:
+            curl.setopt(pycurl.CAINFO, self.cabundle)
 
     def _curl_perform(self, curl, more_headers=[]):
         """Perform curl operation and translate exceptions."""

=== added file 'bzrlib/transport/http/ca_bundle.py'
--- bzrlib/transport/http/ca_bundle.py	1970-01-01 00:00:00 +0000
+++ bzrlib/transport/http/ca_bundle.py	2007-02-26 10:02:58 +0000
@@ -0,0 +1,73 @@
+# Copyright (C) 2007 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Auto-detect of CA bundle for SSL connections"""
+
+import os
+import sys
+from bzrlib.trace import mutter
+
+
+_ca_path = None
+
+
+def get_ca_path(use_cache=True):
+    """Return location of CA bundle"""
+    global _ca_path
+
+    if _ca_path is not None and use_cache:
+        return _ca_path
+
+    # Find CA bundle for SSL
+    # Reimplementation in Python the magic of curl command line tool
+    # from "Details on Server SSL Certificates"
+    # http://curl.haxx.se/docs/sslcerts.html
+    #
+    # 4. If you're using the curl command line tool, you can specify your own CA
+    #    cert path by setting the environment variable CURL_CA_BUNDLE to the path
+    #    of your choice.
+    #
+    #    If you're using the curl command line tool on Windows, curl will search
+    #    for a CA cert file named "curl-ca-bundle.crt" in these directories and in
+    #    this order:
+    #      1. application's directory
+    #      2. current working directory
+    #      3. Windows System directory (e.g. C:\windows\system32)
+    #      4. Windows Directory (e.g. C:\windows)
+    #      5. all directories along %PATH%
+    # bialix: Windows directories usually listed in PATH env variable
+
+    path = os.environ.get('CURL_CA_BUNDLE')
+    if not path and sys.platform == 'win32':
+        dirs = [os.path.realpath(os.path.dirname(sys.argv[0])),     # app dir
+                os.getcwd()]                                        # cwd
+        paths = os.environ.get('PATH')
+        if paths:
+            dirs.extend(paths.split(os.pathsep))
+        for d in dirs:
+            fname = os.path.join(d, "curl-ca-bundle.crt")
+            if os.path.isfile(fname):
+                path = fname
+                break
+    if path:
+        mutter('using CA bundle: %r', path)
+    else:
+        path = ''
+
+    if use_cache:
+        _ca_path = path
+
+    return path

